/*
 * Proiect PM 2021
 * POV display
 * Carapcea Andrei-Antonio 336CA
 */

#include "font.h"
 
#define IC_PIN 8
 
#define FONT_WIDTH 5
 
#define CNT_OVERFLOW 65535
 
#define NO_SEGMENTS 64
 
/*
 * POV display configuration.
 * One rotation is divided into 64 angles
 * At angle i, config[i] is displayed by the LEDs
 * The 8 LEDs are connected to PORTD.
 * LSB = external LED
 * MSB = internal LED
 */
volatile uint8_t config[64];
 
// current index in display configuration
uint8_t curr_idx = 0;
 
// buffer for displaying
volatile byte buffer[64];
 
// current index in display configuration
volatile uint32_t buffer_curr_idx = 0;
 
// Timer1 counter necessary for 1/64 of a rotation
volatile uint32_t segment_cnt = 0;
 
// number of overflows done by Timer1 while counting a rotation period
volatile uint8_t no_overflows = 0;
 
// tell the program to update the display buffer
volatile boolean update_flag = false;
 
// used by Timer1 to only update the buffer once
volatile boolean timer1_updating = false;
 
 
void config_timer1() {
  // normal wave generation
  TCCR1A = 0;
  TCCR1B = 0;
 
  TCNT1 = 0;
  OCR1A = 0xFFFE;

  // ICNC1 = input capture noise canceller
  TCCR1B |= (1 << ICNC1);
 
  // CS10 = prescaler 1
  TCCR1B |= (1 << CS10);
 
  // ICES1 = input capture edge select, 0 = falling edge, 1 = rising edge
  // TCCR1B &= (0 << ICES1);
 
  // input capture interrupt enable
  TIMSK1 = (1 << ICIE1);
 
  // enable output compare A match interrupt
  TIMSK1 |= (1 << OCIE1A);
 
  // timer overflow interrupt enable
  TIMSK1 |= (1 << TOIE1);
}
 
void setup() {
  // disable interrupts
  cli();
 
  // all pins on port D are outputs
  DDRD = 0b11111111;
 
  // initially turn LEDs off
  PORTD = 0;
 
  // set IC_pin as input
  DDRB &= (0 << DDB0);
 
  // enable pullup
  PORTB |= (1 << PORTB0);
 
  config_timer1();
 
  // enable interrupts
  sei();
}
 
void loop()
{
  // clear configuration
  pov_clear();
 
  // write "PROIECT PM" in buffer
  pov_show_string("PM 2021");
 
  // update the display buffer
  pov_update();
 
  // wait
  delay(2000);
 
  // clear configuration
  pov_clear();                 
           
  pov_show_string("POV 2021");
 
  pov_update();               
 
  delay(2000);
}
 
 
// update the display buffer
void pov_update()
{
  update_flag = true;
 
  // wait for display buffer to be updated
  while (update_flag) {
 
  }
}
 
// write a character to the configuration
void pov_show_char(char c)
{
  for (int i = 0; i < FONT_WIDTH + 1; i++) // + 1 because each char has a space
  {
    // update config and go to next column
    config[curr_idx++] = Font8x5[c][i];
  }
}
 
void pov_show_string(String str)
{
  // call pov_show_char for each char in str
  for (char c : str) {
    pov_show_char(c);
  }
}
 
// Clear pov Display
void pov_clear()
{
  for (int i = 0; i < NO_SEGMENTS; ++i)
  {
    config[i] = 0;
  }
  curr_idx = 0;
}
 
// Interrupt overflow Timer1
ISR(TIMER1_OVF_vect)
{
  no_overflows++;
}
 
// Interrupt capture Timer1
ISR(TIMER1_CAPT_vect)
{
   // calculate the counter necessary for 1/64 of a rotation
  segment_cnt = (ICR1 + no_overflows * CNT_OVERFLOW) / NO_SEGMENTS;
 
  // reset buffer display index
  buffer_curr_idx = 0;
 
  // reset timer
  TCNT1 = 0;
 
  // set OCR1A interrupt threshold
  OCR1A = segment_cnt - 1;
 
  // reset no_overflows
  no_overflows = 0;
 
  // only start updating at the start of a rotation
  if (update_flag)
  {
    timer1_updating = true;
  }
}
 
// Interrupt compare OCR1A Timer1
ISR(TIMER1_COMPA_vect)
{
  // update OCR1A to trigger for the next segment
  OCR1A = TCNT1 + segment_cnt - 1;
 
  // if buffer should be updated with new config
  if (timer1_updating)
  {
    // update buffer                                       
    buffer[buffer_curr_idx] = config[buffer_curr_idx];
 
    // if the whole buffer was updated
    if (buffer_curr_idx >= NO_SEGMENTS - 1)
    {
      // mark update as finished
      timer1_updating = false;
      update_flag = false;
    }
  }
 
 // if out of bounds
  if (buffer_curr_idx >= NO_SEGMENTS)
  {
    PORTD = 0;
  }
  else
  {
    // display current LED configuration
    PORTD = buffer[buffer_curr_idx];
  }
 
 
  buffer_curr_idx++; // increment the display column
}
 
 
 
 

